@@ -524,15 +524,25 @@ def propagate_pedal(df, relative=True, drop_pedalend=True, cols={}):
     logger.debug('Extending pedal notes to concerned harmonies')
     beginnings = df.loc[df[pedal].notna(), ['mc', pedal]]
     endings = df.loc[df[pedalend].notna(), ['mc', pedalend]]
-    n_b, n_e = len(beginnings), len(endings)
+    n_beginnings, n_endings = len(beginnings), len(endings)
 
     def make_comparison():
         return pd.concat([beginnings.reset_index(drop=True), endings.reset_index(drop=True)], axis=1).astype({'mc': 'Int64'})
 
-    assert n_b == n_e, f"{n_b} organ points started, {n_e} ended:\n{make_comparison()}"
+    mismatch_maybe_due_to_voltas = None
+    if n_beginnings != n_endings:
+        mismatch_maybe_due_to_voltas = False
+        if 'volta' in df.columns:
+            only_one_volta = (df.volta.fillna(2) == 2)
+            n_endings_cleaned = df.loc[only_one_volta, pedalend].notna().sum()
+            if n_beginnings == n_endings_cleaned:
+                mismatch_maybe_due_to_voltas = True
+                logger.info(f"One or several pedal points have there endings in a first/second ending scenario. "
+                               f"So far I can only correctly propagate the pedal note into first endings, not the others.")
+    if mismatch_maybe_due_to_voltas is False:
+        raise AssertionError(f"{n_beginnings} organ points started, {n_endings} ended:\n{make_comparison()}")
     if relative:
-        assert df[cols[
-            'localkey']].notna().all(), "Local keys must first be propagated using propagate_keys(), no NaNs allowed."
+        assert df[cols['localkey']].notna().all(), "Local keys must first be propagated using propagate_keys(), no NaNs allowed."
 
     for (fro, ped), to in zip(beginnings[pedal].items(), endings[pedalend].index):
         try:
@@ -540,10 +550,17 @@ def propagate_pedal(df, relative=True, drop_pedalend=True, cols={}):
         except:
             logger.error(
                 f"Slicing of the DataFrame did not work from {fro} to {to}. Index looks like this:\n{df.head().index}")
-        localkeys = df.loc[section, cols['localkey']]
+        section_df = df.loc[section]
+        if mismatch_maybe_due_to_voltas and section_df['volta'].notna().any():
+            only_one_volta = section_df.volta.fillna(1) == 1
+            if not only_one_volta.all():
+                # ToDo: Make full-fledged solution for correct propagation to endings in several voltas or even beyond
+                section = section[only_one_volta]
+                section_df = df.loc[section]
+        localkeys = section_df[cols['localkey']]
         if localkeys.nunique() > 1:
             first_localkey = localkeys.iloc[0]
-            globalkeys = df.loc[section, cols['globalkey']].unique()
+            globalkeys = section_df[cols['globalkey']].unique()
             assert len(globalkeys) == 1, "Several globalkeys appearing within the same organ point."
             global_minor = globalkeys[0].islower()
             # if the localkey changes during the pedal point, the reference changes and the Roman numeral indicating
